package com.persagy.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.googlecode.aviator.AviatorEvaluator;
import com.googlecode.aviator.Expression;
import com.persagy.cache.AlarmInfoCache;
import com.persagy.cache.CurrentDataCache;
import com.persagy.client.GroupNettyClient;
import com.persagy.entity.*;
import com.persagy.service.AlarmHandleService;
import com.persagy.utils.DateUtils;
import com.persagy.utils.StringUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Objects;

/**
 * @description:报警处理实现类：包含报警产生和报警恢复
 * @author:LuoGuanhyi
 * @company:PersagyTechnologyCo.,Ltd
 * @since:2020/10/15 001511:14
 * @version:V1.0
 **/
@Service
@Slf4j
public class AlarmHandleServiceImpl implements AlarmHandleService {
    @Autowired
    AlarmInfoCache alarmInfoCache;
    @Autowired
    CurrentDataCache currentDataCache;
    @Autowired
    GroupNettyClient groupNettyClient;
    /**
     * 集团编码
     */
    @Value("${group.code}")
    public String groupCode;
    /**
     * 系统标识，用于创建人
     */
    @Value("${system.id}")
    public String systemId;

    @Override
    public void handleIOTData(String msg) {
        String[] split = msg.split(";");
        if (split.length < 4) {
            return;
        }
        //返回的信息点个数
        int nums = (split.length) / 4;
        System.out.println("length:"+split.length);
        for (int i = 0; i < nums; i++) {
            String dateTime = split[i * 4];
            String meterId = split[i * 4 + 1];
            String funcId = split[i * 4 + 2];
            double value = Double.parseDouble(split[i * 4 + 3]);

            if (alarmInfoCache.hasKey(meterId, funcId)) {
                currentDataCache.putCurrentData(meterId, funcId, value);
                List<AlarmDefine> alarmDefines = alarmInfoCache.getAlarmDefinitionIdByMeterFuncId(meterId, funcId);
                for (AlarmDefine alarmDefine : alarmDefines) {
                    Condition condition = alarmDefine.getCondition();
                    List<JSONObject> codeDetail = condition.getInfoCodes();
                    boolean match = codeDetail.stream().allMatch(p -> currentDataCache.hasKey(p.getString("meterId"), p.getString("funcId")));
                    if (match) {
                        String defineId = alarmInfoCache.getAlarmDefineId(alarmDefine);
                        String trigger = condition.getTrigger();
                        String end = condition.getEnd();
                        HashMap paramMap = new HashMap<>();
                        for (int j = 0; j < codeDetail.size(); j++) {
                            JSONObject code = codeDetail.get(j);
                            paramMap.put(code.getString("infoCode"), currentDataCache.getCurrentData(code.getString("meterId"), code.getString("funcId")));
                        }
                        Expression triggerExp = AviatorEvaluator.compile(trigger, true);
                        Expression endExp = AviatorEvaluator.compile(end, true);
                        Boolean triggerResult = (Boolean) triggerExp.execute(paramMap);
                        Boolean endResult = (Boolean) endExp.execute(paramMap);
                        System.out.println("triggerResult:" + triggerResult);
                        System.out.println("endResult:" + endResult);
                        AlarmState alarmState = alarmInfoCache.getAlarmState(defineId);
                        if (Objects.isNull(alarmState)) {
                            alarmState = new AlarmState(defineId);
                        }
                        //报警产生值满足（这里的满足不考虑报警持续时间）
                        if (triggerResult) {
                            //报警的时候不考虑报警恢复，因为同时报警和报警恢复是不应该出现的
                            handlerNowDataAlarm(alarmDefine,alarmState, dateTime, condition, defineId);
                        } else {
                            //当前数据正常
                            handlerNowDataNormal(alarmDefine, dateTime, condition, defineId, endResult, alarmState);
                        }
                    } else {
                        log.warn("部分信息点没有数值:[{}]", codeDetail);
                    }
                }
            }

        }
    }
    /**
     * @description:当前数据正常判断逻辑
     * @exception:
     * @author: LuoGuangyi
     * @company: Persagy Technology Co.,Ltd
     * @param alarmDefine: 报警定义
     * @param dateTime:IOT采集时间
     * @param condition:报警条件
     * @param defineId:报警定义ID
     * @param endResult:报警恢复监测结果
     * @param alarmState: 报警状态时间实体
     * @return: void
     * @since: 2020/10/21 16:46
     * @version: V1.0
     */
    private void handlerNowDataNormal(AlarmDefine alarmDefine, String dateTime, Condition condition, String defineId, Boolean endResult, AlarmState alarmState) {
        //当前数据正常，报警状态为正常：清空之前的报警计时，重置回默认状态
        if (AlarmState.State.NORMAL.getType().equals(alarmState.getState())){
            alarmState = new AlarmState(defineId);
            alarmInfoCache.setAlarmState(defineId, alarmState);
        }else if(AlarmState.State.NOT_DEAL.getType().equals(alarmState.getState())){
            //报警状态异常时候
            alarmState.setLatestDataNormalstate(true);
            //报警恢复条件满足，判断是否满足报警恢复持续时间
            if(endResult){
                String endTime = alarmState.getAlarmEndTime();
                if(StringUtil.isEmpty(endTime)){
                    endTime = dateTime;
                }
                alarmState.setAlarmEndTime(endTime);
                alarmState.setAlarmStartTime("");
                //设置开始恢复时间
                int uphold = condition.getEndUphold();
                //超过报警恢复设置的持续时间
                if(DateUtils.betweenTwoTimeSecond(endTime, dateTime)>=0){
                    log.error("产生一条报警恢复消息");
                    log.error("产生一条取消报警过期定时任务的消息");
                    //报警恢复，报警状态重置回默认
                    alarmState.reset();
                }
            }
            alarmInfoCache.setAlarmState(defineId, alarmState);
        }
    }

    /**
     *
     * @param alarmDefine 报警定义
     * @param dateTime :IOT数据采集时间
     * @param condition :报警定义条件
     * @param defineId :报警定义ID
     * @description:处理当前值报警的情况
     * @exception:
     * @author: LuoGuangyi
     * @company: Persagy Technology Co.,Ltd
     * @return: void
     * @since: 2020/10/20 22:31
     * @version: V1.0
     */
    private void handlerNowDataAlarm(AlarmDefine alarmDefine, AlarmState alarmState, String dateTime, Condition condition, String defineId) {
        alarmState.setLatestDataNormalstate(false);
        JSONObject time = condition.getTime();
        boolean inEffectTime = false;
        boolean hasExpire = true;
        //生效时间段，有误过期时间判断
        if (StringUtil.isEmpty(time) || StringUtil.isEmpty(time.getString("type"))) {
            //无生效时间设置，则任何时间生效
            inEffectTime = true;
            //没有设置生效时间，所以也没有设置过期时间
            hasExpire = false;

        }else {
            if("period".equals(time.getString("type")) && "0000".equals(time.getString("$gt").substring(0,4)) && "2359".equals(time.getString("$lt").substring(0,4))){
                //架构师w确认 00:00-23:59的生效时间为一值生效，23:59分不过期
                hasExpire = false;
            }
            if ("period".equals(time.getString("type")) && time.getString("$gt").compareTo(DateUtils.getSdfTimeNotDate(dateTime)) >= 0 && time.getString("$lt").compareTo(DateUtils.getSdfTimeNotDate(dateTime)) <= 0) {
                //在生效时间
                inEffectTime = true;
            } else {
                log.info("[{}]不在设置的生效时间段", dateTime);
            }
        }
        if (inEffectTime) {
            //之前是报警状态，现在还是报警,报警值没变化
            //之前是是正常时候
            if (AlarmState.State.NORMAL.getType().equals(alarmState.getState())) {
                //设置开始报警时间
                alarmState.setAlarmStartTime(dateTime);
                long timeSecond = 0;
                if (StringUtil.isNotEmpty(alarmState.getAlarmStartTime())) {
                    timeSecond = DateUtils.betweenTwoTimeSecond(alarmState.getAlarmStartTime(), dateTime);
                }
                if (timeSecond >= condition.getTriggerUphold()) {
                    log.error("大于持续时间了，产生一条报警");
                    AlarmRecord alarmRecord = AlarmRecord.builder()
                            .groupCode(groupCode)
                            .itemCode(alarmDefine.getItemCode())
                            .createUser(systemId)
                            .createTime(DateUtils.getMilliByLocalDateTime(dateTime))
                            .objId(alarmDefine.getObjId())
                            .build();
                    NettyMessage<AlarmRecord> nettyMessage = new NettyMessage<>();
                    nettyMessage.setOpCode(1);
                    nettyMessage.setContent(Arrays.asList(alarmRecord));
                    //推送一条报警记录给远端
                    groupNettyClient.sendMessage(nettyMessage.toString());
                    alarmState.setState(AlarmState.State.NOT_DEAL.getType());
                    //有过期时间，生成报警过期消息
                    if(hasExpire && "period".equals(time.getString("type"))){
                        //过期时间
                        String expireTime = time.getString("$lt");
                        log.error("产生一条定时过期报警消息");

                    }
                } else {
                    alarmState.setState(AlarmState.State.NORMAL.getType());
                }
            }
        }else {
            //不在生效时间的段的产生报警,要保存最近一次的数值是否是是报警
            //其他条件全部改成默认值（不报警，不过期，报警开始时间和结束时间为空）
            alarmState.reset();
        }
        alarmInfoCache.setAlarmState(defineId, alarmState);
    }
}
